@@ -5,9 +5,13 @@
 #include "pac_output.h"
 #include "pac_typedecl.h"
 #include "pac_utils.h"
+#include "pac_btype.h"
 
 #include "pac_case.h"
 
+#include <limits>
+#include <stdint.h>
+
 CaseType::CaseType(Expr* index_expr, CaseFieldList* cases)
 	: Type(CASE), index_expr_(index_expr), cases_(cases)
 	{
@@ -64,7 +68,9 @@ void CaseType::Prepare(Env* env, int flags)
 	ASSERT(flags & TO_BE_PARSED);
 
 	index_var_ = new ID(strfmt("%s_case_index", value_var()->Name()));
-	env->AddID(index_var_, MEMBER_VAR, extern_type_int);
+	// Unable to get the type for index_var_ at this moment, but we'll
+	// generate the right type based on index_expr_ later.
+	env->AddID(index_var_, MEMBER_VAR, 0);
 
 	// Sort the cases_ to put the default case at the end of the list
 	CaseFieldList::iterator default_case_it =
@@ -100,13 +106,25 @@ void CaseType::Prepare(Env* env, int flags)
 
 void CaseType::GenPrivDecls(Output* out_h, Env* env)
 	{
-	out_h->println("int %s;", env->LValue(index_var_));
+	Type* t = index_expr_->DataType(env);
+
+	if ( t->tot() != Type::BUILTIN )
+		// It's a Type::EXTERN with a C++ type of "int", "bool", or "enum",
+		// any of which will convert consistently using an int as storage type.
+		t = extern_type_int;
+
+	out_h->println("%s %s;", t->DataTypeStr().c_str(), env->LValue(index_var_));
 	Type::GenPrivDecls(out_h, env);
 	}
 
 void CaseType::GenPubDecls(Output* out_h, Env* env)
 	{
-	out_h->println("int %s const	{ return %s; }",
+	Type* t = index_expr_->DataType(env);
+
+	if ( t->tot() != Type::BUILTIN )
+		t = extern_type_int;
+
+	out_h->println("%s %s const	{ return %s; }", t->DataTypeStr().c_str(),
 		env->RValue(index_var_), env->LValue(index_var_));
 	Type::GenPubDecls(out_h, env);
 	}
@@ -168,7 +186,7 @@ void CaseType::DoGenParseCode(Output* out_cc, Env* env,
 		{
 		out_cc->println("default:");
 		out_cc->inc_indent();
-		out_cc->println("throw binpac::ExceptionInvalidCaseIndex(\"%s\", %s);",
+		out_cc->println("throw binpac::ExceptionInvalidCaseIndex(\"%s\", (int64)%s);",
 			decl_id()->Name(), env->RValue(index_var_));
 		out_cc->println("break;");
 		out_cc->dec_indent();
@@ -248,18 +266,84 @@ CaseField::~CaseField()
 	delete_list(ExprList, index_);
 	}
 
-void GenCaseStr(ExprList *index_list, Output *out_cc, Env *env)
+void GenCaseStr(ExprList *index_list, Output *out_cc, Env *env, Type* switch_type)
 	{
 	if ( index_list )
 		{
 		foreach(i, ExprList, index_list)
 			{
 			Expr *index_expr = *i;
+
+			Type* case_type = index_expr->DataType(env);
+
+			if ( case_type->tot() == Type::BUILTIN && case_type->StaticSize(env) > 4 )
+				throw ExceptionInvalidCaseSizeExpr(index_expr);
+
 			int index_const;
 
 			if ( ! index_expr->ConstFold(env, &index_const) )
 				throw ExceptionNonConstExpr(index_expr);
-			out_cc->println("case %d:", index_const);
+
+			// External C++ types like "int", "bool", "enum"
+			// all use "int" type internally by default.
+			int case_type_width = 4;
+			int switch_type_width = 4;
+
+			if ( switch_type->tot() == Type::BUILTIN )
+				switch_type_width = switch_type->StaticSize(env);
+
+			if ( case_type->tot() == Type::BUILTIN )
+				case_type_width = case_type->StaticSize(env);
+
+			if ( case_type_width > switch_type_width )
+				{
+				BuiltInType* st = (BuiltInType*)switch_type;
+
+				if ( switch_type_width == 1 )
+					{
+					if ( st->bit_type() == BuiltInType::INT8 )
+						{
+						if ( index_const < std::numeric_limits<int8_t>::min() )
+							throw ExceptionInvalidCaseLimitExpr(index_expr);
+						if ( index_const > std::numeric_limits<int8_t>::max() )
+							throw ExceptionInvalidCaseLimitExpr(index_expr);
+						}
+					else
+						{
+						if ( index_const < std::numeric_limits<uint8_t>::min() )
+							throw ExceptionInvalidCaseLimitExpr(index_expr);
+						if ( index_const > std::numeric_limits<uint8_t>::max() )
+							throw ExceptionInvalidCaseLimitExpr(index_expr);
+						}
+					}
+				else if ( switch_type_width == 2 )
+					{
+					if ( st->bit_type() == BuiltInType::INT16 )
+						{
+						if ( index_const < std::numeric_limits<int16_t>::min() )
+							throw ExceptionInvalidCaseLimitExpr(index_expr);
+						if ( index_const > std::numeric_limits<int16_t>::max() )
+							throw ExceptionInvalidCaseLimitExpr(index_expr);
+						}
+					else
+						{
+						if ( index_const < std::numeric_limits<uint16_t>::min() )
+							throw ExceptionInvalidCaseLimitExpr(index_expr);
+						if ( index_const > std::numeric_limits<uint16_t>::max() )
+							throw ExceptionInvalidCaseLimitExpr(index_expr);
+						}
+					}
+				else
+					{
+					assert(0);
+					}
+				}
+
+			// We're always using "int" for storage, so ok to just
+			// cast into the type used by the switch statement since
+			// some unsafe stuff is already checked above.
+			out_cc->println("case ((%s) %d):",
+			                switch_type->DataTypeStr().c_str(), index_const);
 			}
 		}
 	else
@@ -296,7 +380,7 @@ void CaseField::GenPubDecls(Output* out_h, Env* env)
 		out_h->println("switch ( %s )", env->RValue(index_var_));
 		out_h->inc_indent();
 		out_h->println("{");
-		GenCaseStr(index_, out_h, env);
+		GenCaseStr(index_, out_h, env, case_type()->IndexExpr()->DataType(env));
 		out_h->inc_indent();
 		out_h->println("break;  // OK");
 		out_h->dec_indent();
@@ -304,7 +388,7 @@ void CaseField::GenPubDecls(Output* out_h, Env* env)
 		out_h->println("default:");
 		out_h->inc_indent();
 		out_h->println(
-			"throw binpac::ExceptionInvalidCase(\"%s\", %s, \"%s\");",
+			"throw binpac::ExceptionInvalidCase(\"%s\", (int64)%s, \"%s\");",
 			id_->LocName(),
 			env->RValue(index_var_),
 			OrigExprList(index_).c_str());
@@ -335,7 +419,7 @@ void CaseField::GenInitCode(Output* out_cc, Env* env)
 
 void CaseField::GenCleanUpCode(Output* out_cc, Env* env)
 	{
-	GenCaseStr(index_, out_cc, env);
+	GenCaseStr(index_, out_cc, env, case_type()->IndexExpr()->DataType(env));
 	out_cc->inc_indent();
 	out_cc->println("// Clean up \"%s\"", id_->Name());
 	out_cc->println("{");
@@ -349,29 +433,28 @@ void CaseField::GenCleanUpCode(Output* out_cc, Env* env)
 void CaseField::GenParseCode(Output* out_cc, Env* env,
 		const DataPtr& data, const ID* size_var)
 	{
-	GenCaseStr(index_, out_cc, env);
+	GenCaseStr(index_, out_cc, env, case_type()->IndexExpr()->DataType(env));
 	out_cc->inc_indent();
 	out_cc->println("// Parse \"%s\"", id_->Name());
 	out_cc->println("{");
 
 	{
 	Env case_env(env, this);
-	Env *env = &case_env;
 
-	type_->GenPreParsing(out_cc, env);
-	type_->GenParseCode(out_cc, env, data, 0);
+	type_->GenPreParsing(out_cc, &case_env);
+	type_->GenParseCode(out_cc, &case_env, data, 0);
 	if ( size_var )
 		{
 		out_cc->println("%s = %s;",
-			env->LValue(size_var),
-			type_->DataSize(out_cc, env, data).c_str());
+			case_env.LValue(size_var),
+			type_->DataSize(out_cc, &case_env, data).c_str());
 		}
 	if ( type_->incremental_input() )
 		{
 		ASSERT(case_type()->parsing_complete_var());
 		out_cc->println("%s = %s;",
-			env->LValue(case_type()->parsing_complete_var()),
-			env->RValue(type_->parsing_complete_var()));
+			case_env.LValue(case_type()->parsing_complete_var()),
+			case_env.RValue(type_->parsing_complete_var()));
 		}
 	out_cc->println("}");
 	}
