@@ -1,6 +1,7 @@
 #include "pac_exttype.h"
 #include "pac_id.h"
 #include "pac_decl.h"
+#include "pac_output.h"
 
 bool ExternType::DefineValueVar() const
 	{
@@ -8,14 +9,14 @@ bool ExternType::DefineValueVar() const
 	}
 
 string ExternType::DataTypeStr() const
-	{ 
+	{
 	switch ( ext_type_ )
 		{
 		case PLAIN:
 		case NUMBER:
-			return id_->Name(); 
+			return id_->Name();
 		case POINTER:
-			return string(id_->Name()) + " *"; 
+			return string(id_->Name()) + " *";
 		default:
 			ASSERT(0);
 			return "";
@@ -28,18 +29,26 @@ int ExternType::StaticSize(Env* env) const
 	return -1;
 	}
 
-bool ExternType::ByteOrderSensitive() const 
-	{ 
+bool ExternType::ByteOrderSensitive() const
+	{
 	return false;
 	}
 
 string ExternType::EvalMember(const ID *member_id) const
 	{
-	return strfmt("%s%s", 
+	return strfmt("%s%s",
 		ext_type_ == POINTER ? "->" : ".",
 		member_id->Name());
 	}
 
+void ExternType::GenInitCode(Output* out_cc, Env* env)
+	{
+	if ( IsNumericType() || IsPointerType() )
+		out_cc->println("%s = 0;", env->LValue(value_var()));
+
+	Type::GenInitCode(out_cc, env);
+	}
+
 void ExternType::DoGenParseCode(Output* out, Env* env, const DataPtr& data, int flags)
 	{
 	ASSERT(0);
@@ -51,8 +60,8 @@ void ExternType::GenDynamicSize(Output* out, Env* env, const DataPtr& data)
 	}
 
 Type *ExternType::DoClone() const
-	{ 
-	return new ExternType(id_->clone(), ext_type_); 
+	{
+	return new ExternType(id_->clone(), ext_type_);
 	}
 
 // Definitions of pre-defined external types
@@ -70,7 +79,7 @@ void ExternType::static_init()
 #define EXTERNTYPE(name, ctype, exttype) \
 	id = new ID(#ctype); \
 	extern_type_##name = new ExternType(id, ExternType::exttype); \
-	Type::AddPredefinedType(#name, extern_type_##name); 
+	Type::AddPredefinedType(#name, extern_type_##name);
 #include "pac_externtype.def"
 #undef EXTERNTYPE
 	}
